@@ -1,19 +1,19 @@
 /*
-    A widget to display and manipulate tabular data
-    Copyright (C) 2017, 2020  John Darrington
+  A widget to display and manipulate tabular data
+  Copyright (C) 2017, 2020  John Darrington
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
+  This program is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <config.h>
@@ -22,15 +22,15 @@
 #include "ssw-paste.h"
 
 static void
-start_element  (GMarkupParseContext *context,
-		const gchar         *element_name,
-		const gchar        **attribute_names,
-		const gchar        **attribute_values,
-		gpointer             user_data,
-		GError             **error)
+start_element (GMarkupParseContext *context,
+               const gchar *element_name,
+               const gchar **attribute_names,
+               const gchar **attribute_values,
+               gpointer user_data,
+               GError **error)
 {
   struct paste_state *state = user_data;
-  
+
   if (0 == g_ascii_strcasecmp ("table", element_name))
     {
       state->row = 0;
@@ -48,10 +48,10 @@ start_element  (GMarkupParseContext *context,
 }
 
 static void
-end_element    (GMarkupParseContext *context,
-		const gchar         *element_name,
-		gpointer             user_data,
-		GError             **error)
+end_element (GMarkupParseContext *context,
+             const gchar *element_name,
+             gpointer user_data,
+             GError **error)
 {
   struct paste_state *state = user_data;
 
@@ -72,12 +72,12 @@ end_element    (GMarkupParseContext *context,
     }
 }
 
-static  void
-text           (GMarkupParseContext *context,
-		const gchar         *text,
-		gsize                text_len,
-		gpointer             user_data,
-		GError             **error)
+static void
+text_i (GMarkupParseContext *context,
+      const gchar *text,
+      gsize text_len,
+      gpointer user_data,
+      GError **error)
 {
   struct paste_state *state = user_data;
   if (state->cell_element)
@@ -90,10 +90,10 @@ text           (GMarkupParseContext *context,
 
 static const GMarkupParser my_parser =
   {
-    start_element,
-    end_element,
-    text,
-    0
+   start_element,
+   end_element,
+   text_i,
+   0
   };
 
 
@@ -115,7 +115,6 @@ html_parse (GtkClipboard *clip, GtkSelectionData *sd, gpointer user_data)
   GMarkupParseContext *ctx = g_markup_parse_context_new (&my_parser, 0, ps, 0);
   g_markup_parse_context_parse (ctx, (const gchar *) text, len, 0);
   g_markup_parse_context_unref (ctx);
-  
+
   gtk_widget_queue_draw (GTK_WIDGET (sheet));
 }
-
